#!/usr/bin/perl 
 
# 
#  FASt Term Recognizer
# 
#   DISTRIB/bin/TreeTaggertoFastr.pl
#   Version 2.04/04-02-01
# 
#   Copyright (C) 2004  Christian Jacquemin, LIMSI-CNRS
# 
#   Conversion of a corpus tagged by TreeTagger 
#   into the feature-based format of FASTR
# 

# STDIN: A corpus tagged by TreeTagger (tree-tagger-english)
#
# <FIC NUM="9404001">
# Since   IN      since
# large   JJ      large
# amounts NNS     amount
# of      IN      of
# textual JJ      textual

# IN: FILETags = file for tag conversion
#
# #
#    <cat> = Pu.\n
# # punctuation
# $
#    <cat> = Pu.\n
# # punctuation
# ''
#    <cat> = Pu.\n
# # punctuation

# STDOUT: a tagged corpus into FASTR format
#
# '<FIC NUM="9404001">'
#    <cat> = St.
# 'Since'
#    <lem> = 'since'
#    <cat> = P.
# 'large'
#    <lem> = 'large'
#    <cat> = A.
# 'amounts'
#    <lem> = 'amount'
#    <cat> = N
#    <agreement number> = plural.

# syntactic check
if( (@ARGV + 0) != 1 ) {
    die "Usage: TreeTaggertoFastr.pl <tag-file>!\n";
}

# reading the extended tags
$FILENAMETAGS = shift @ARGV;
open( FILETags , "<$FILENAMETAGS" ) 
    ||  die "Can't open file ($FILENAMETAGS)!" ;

%TAGEQUIVALENCE = ();
while( <FILETags> ){
    # TreeTagger tag
    chop;
    $TAGORIG = $_;
    if( !($_ = <FILETags>) ){
	break;
    }

    # fastr tag / features
    chop;
    $NEWTAG = $_;

    # comments
    if( !<FILETags> ){
	break;
    }

    # transforms \\n into \n and 
    # stores the translation
    $NEWTAG =~ s/\\n/\n/g;
    $TAGEQUIVALENCE{ $TAGORIG } = $NEWTAG;
}
close( FILETags );

# converting the tagged file
while (<>) {
    chop;

    # empty sentence or empty term description
    if( /^$/ ){
    }

    # document identifier
    if( /^\<.*\>$/ ){
	$WORD = $_;
	$NEWTAG = $TAGEQUIVALENCE{ "Struct" };
	if( $NEWTAG eq "" ){
	    print STDERR "***** Unknown tag [$WORD/Struct]\n";
	} 
	else {
	    # prints features of doct structure
	    # backslashing of '
	    $_ =~ s/\'/\\\'/g;
	    $_ =~ s/\t/ /g;
	    print "\'$_\'\n$NEWTAG";
	}
    }

    # splitting the line into: inflected word, tag, and lemma
    if( !( /^\<.*\>$/ )  && !( /^$/ ) ){
	( $INFLWORD , $TAG , $LEMMA ) = split( /\t/ );

	# unknown lemmas are replaced by the inflected word
	if( $LEMMA eq "<unknown>" || $LEMMA eq "\@card\@" ) {
	    $LEMMA = $INFLWORD;
	}

        # normalizing words
	if( !($TAG =~ /[\#\$\'()\.:;,]/ || $TAG =~ /PON/ ) ) {
	    &CleanWord;
	}

        # tag translation
	$NEWTAG = $TAGEQUIVALENCE{ $TAG };
	if( $NEWTAG eq "" ){
	    print STDERR "***** Unknown tag [$INFLWORD/$TAG]\n";
	} else  {
	    # prints words and features
	    # backslashing of '
	    $INFLWORD =~ s/\\/\\\\/g;
	    $LEMMA =~ s/\\/\\\\/g;
	    $INFLWORD =~ s/\'/\\\'/g;
	    $LEMMA =~ s/\'/\\\'/g;
	    print "\'$INFLWORD\'\n   <lem> = \'$LEMMA\'\n$NEWTAG";
	}
    }
}

sub CleanWord {
    # suppression of parasite characters
    $INFLWORD =~ s/[\"\{\}\$\~\=\|*,\\\]\[\`\<\>\?\@\_]//g;
    # deletion of double quotes
    $INFLWORD =~ s/\'\'//g;
    # deletion of initial and final single quote
    $INFLWORD =~ s/^\'//;
    $INFLWORD =~ s/\'$//;
    # normalization of double or triple hyphens
    $INFLWORD =~ s/---/-/g;
    $INFLWORD =~ s/--/-/g;
    # deletion of initial and final  hyphen
    $INFLWORD =~ s/^-//;
    $INFLWORD =~ s/-$//;
    # suppression of tabulations
    $INFLWORD =~ s/\t/ /g;

    # suppression of parasite characters
    $LEMMA =~ s/[\"\{\}\$\~\=\|*,\\\]\[\`\<\>\?\@\_]//g;
    # deletion of double quotes
    $LEMMA =~ s/\'\'//g;
    # deletion of initial and final single quote
    $LEMMA =~ s/^\'//;
    $LEMMA =~ s/\'$//;
    # normalization of double or triple hyphens
    $LEMMA =~ s/---/-/g;
    $LEMMA =~ s/--/-/g;
    # deletion of initial and final  hyphen
    $LEMMA =~ s/^-//;
    $LEMMA =~ s/-$//;
    # suppression of tabulations
    $LEMMA =~ s/\t/ /g;
}
