#!/usr/bin/perl 

# 
#  FASt Term Recognizer
# 
#   DISTRIB/bin/WordNetPreProc.pl
#   Version 2.04/04-02-01
# 
#   Copyright (C) 2004  Christian Jacquemin, LIMSI-CNRS
# 
#   Pre-processing of the WordNet database for creating semantic families
# 

# STDIN: WORDNET original list of synsets
# 
# Synonyms/Hypernyms (Ordered by Frequency) of verb abandon
# 
# 5 senses of abandon                                                     
# 
# Sense 1
# abandon
#        => discard, fling, toss, toss out, toss away, chuck out, 
#           cast aside, dispose, throw out, cast out, throw away, 
#           cast away, put away
# 
# Sense 2
# abandon, give up

# STDOUT: semantic families for fastr
#
# abandon\V\: abandon\V\ vacate\V\ empty\V\ give\V\ forsake\V\ 
#             desolate\V\ desert\V\ lurch\V\
# abase\V\: humiliate\V\ mortify\V\ chagrin\V\ humble\V\ abase\V\

# syntactic check
if( (@ARGV + 0) != 1 ) {
    die "Usage: WordNetPreProc.pl [-nocompound|-compound]!\n";
}

if( @ARGV[0] eq "-nocompound" ) {
    $withcompound = 0;
}
elsif( @ARGV[0] eq "-nocompound" ) {
    $withcompound = 1;
}
else {
    die "Usage: WordNetPreProc.pl [-nocompound|-compound]!\n";
}

sub PrintSynset {
    # replaces spaces by undescores in the compounds
    $synset =~ s/([^,]) /$1\_/g;

    # suppresses compounds in case of -nocompound option
    if( !$withcompound ) {
	$synset =~ s/[^, ]+\_[^, ]+, //g;
    }
    # prints the synset
    @synset = split( /, / , $synset );

    # only synsets of non compound lemmas are printed
    # in case of -nocompound option
    if( $withcompound || !($headlemma =~ /\_/) ) {
	$outputString = "$headlemma\\$headcat\\:";
	%knownTail = ();
	$numberOfSynnonyms = 0;
	foreach $synonym ( @synset ) {
	    if( !$knownTail{ $synonym } ) {
	        $outputString = "$outputString $synonym\\$headcat\\";
		$knownTail{ $synonym } = 1;		
		$numberOfSynnonyms++;
	    }
	}
	# only synsets of more than one word are printed
	if( $numberOfSynnonyms >= 2 ) {
	    print STDOUT "$outputString\n";
	}
    }
}

# intialize the first synset
$synset = "";
while (<STDIN>) {
    if( /Synonyms.*of (.*) (.*)$/ || /Similarity.*of (.+) (.+)$/ ) {
	$newheadcat = $1;
	$newheadlemma = $2;

	&PrintSynset;
	$synset = "";

	$headcat = $newheadcat;
	$headlemma = $newheadlemma;
	if( $headcat eq "noun" ) {
	    $headcat = "N";
	}
	elsif( $headcat eq "verb" ) {
	    $headcat = "V";
	}
	elsif( $headcat eq "adv" ) {
	    $headcat = "ADV";
	}
	elsif( $headcat eq "adj" ) {
	    $headcat = "A";
	}
	else {
	    die "Unknown category $headcat !";
	}
    }

    # writing the semantic family (the synset)
    if( /^Sense/ ) {
	$lineSynSet = <STDIN>;
	chop( $lineSynSet );
	$synset = $synset . $lineSynSet . ", ";
    }
}
&PrintSynset;

